"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const writeFileSafely_1 = require("./utils/writeFileSafely");
class Transformer {
    constructor({ name, fields, modelOperations, enumTypes, }) {
        this.name = name !== null && name !== void 0 ? name : '';
        this.fields = fields !== null && fields !== void 0 ? fields : [];
        this.modelOperations = modelOperations !== null && modelOperations !== void 0 ? modelOperations : [];
        this.schemaImports = new Set();
        this.enumTypes = enumTypes;
    }
    static setOutputPath(outPath) {
        this.outputPath = outPath;
    }
    static getOutputPath() {
        return this.outputPath;
    }
    addSchemaImport(name) {
        var _a;
        (_a = this.schemaImports) === null || _a === void 0 ? void 0 : _a.add(name);
    }
    getAllSchemaImports() {
        var _a;
        return [...((_a = this.schemaImports) !== null && _a !== void 0 ? _a : [])]
            .map((name) => name === 'SortOrder'
            ? `import { ${name}Schema } from '../enums/${name}.schema'`
            : `import { ${name}SchemaObject } from './${name}.schema'`)
            .join(';\r\n');
    }
    getPrismaStringLine(field, inputType, inputsLength) {
        if (inputsLength === 1) {
            if (inputType.isList) {
                if (inputType.type === this.name) {
                    return `  ${field.name}: Joi.array().items(${`Joi.link('#${inputType.type}')`})`;
                }
                else {
                    return `  ${field.name}: ${inputType.type === 'SortOrder'
                        ? `${`${inputType.type}Schema`}`
                        : `Joi.array().items(Joi.object().keys(${`${inputType.type}SchemaObject`}))`}`;
                }
            }
            else {
                if (inputType.type === this.name) {
                    return `  ${field.name}: ${`Joi.link('#${inputType.type}')`}`;
                }
                else {
                    return `  ${field.name}: ${inputType.type === 'SortOrder'
                        ? `${`${inputType.type}Schema`}`
                        : `Joi.object().keys(${`${inputType.type}SchemaObject`})`}`;
                }
            }
        }
        if (inputsLength > 1) {
            if (inputType.isList) {
                if (inputType.type === this.name) {
                    return `Joi.array().items(${`Joi.link('#${inputType.type}')`})`;
                }
                else {
                    return `${inputType.type === 'SortOrder'
                        ? `${`${inputType.type}Schema`}`
                        : `Joi.array().items(Joi.object().keys(${`${inputType.type}SchemaObject`}))`}`;
                }
            }
            else {
                if (inputType.type === this.name) {
                    return `${`Joi.link('#${inputType.type}')`}`;
                }
                else {
                    return `${inputType.type === 'SortOrder'
                        ? `${`${inputType.type}Schema`}`
                        : `Joi.object().keys(${`${inputType.type}SchemaObject`})`}`;
                }
            }
        }
        return '';
    }
    getSchemaObjectLine(field) {
        let lines = field.inputTypes;
        const inputsLength = field.inputTypes.length;
        if (inputsLength === 0)
            return lines;
        if (inputsLength === 1) {
            lines = lines.map((inputType) => {
                if (inputType.type === 'String') {
                    return [
                        `  ${field.name}: ${inputType.isList
                            ? 'Joi.array().items(Joi.string())'
                            : 'Joi.string()'}`,
                        field,
                    ];
                }
                else if (inputType.type === 'Int' || inputType.type === 'Float') {
                    return [
                        `  ${field.name}: ${inputType.isList
                            ? 'Joi.array().items(Joi.number())'
                            : 'Joi.number()'}`,
                        field,
                    ];
                }
                else if (inputType.type === 'Boolean') {
                    return [
                        `  ${field.name}: ${inputType.isList
                            ? 'Joi.array().items(Joi.boolean())'
                            : 'Joi.boolean()'}`,
                        field,
                    ];
                }
                else if (inputType.type === 'DateTime') {
                    return [
                        `  ${field.name}: ${inputType.isList ? 'Joi.array().items(Joi.date())' : 'Joi.date()'}`,
                        field,
                    ];
                }
                else {
                    if (inputType.namespace === 'prisma') {
                        if (inputType.type !== this.name) {
                            this.addSchemaImport(inputType.type);
                        }
                        return [
                            this.getPrismaStringLine(field, inputType, inputsLength),
                            field,
                            true,
                        ];
                    }
                }
                return [];
            });
        }
        else {
            const alternatives = lines.reduce((result, inputType) => {
                if (inputType.type === 'String') {
                    result.push(inputType.isList
                        ? 'Joi.array().items(Joi.string())'
                        : 'Joi.string()');
                }
                else if (inputType.type === 'Int' || inputType.type === 'Float') {
                    result.push(inputType.isList
                        ? 'Joi.array().items(Joi.number())'
                        : 'Joi.number()');
                }
                else if (inputType.type === 'Boolean') {
                    result.push(inputType.isList
                        ? 'Joi.array().items(Joi.boolean())'
                        : 'Joi.boolean()');
                }
                else {
                    if (inputType.namespace === 'prisma') {
                        if (inputType.type !== this.name) {
                            this.addSchemaImport(inputType.type);
                        }
                        result.push(this.getPrismaStringLine(field, inputType, inputsLength));
                    }
                }
                return result;
            }, []);
            if (alternatives.length > 0) {
                lines = [
                    [
                        `  ${field.name}: Joi.alternatives().try(${alternatives.join(',\r\n')})`,
                        field,
                        true,
                    ],
                ];
            }
            else {
                return [[]];
            }
        }
        return lines.filter(Boolean);
    }
    getFieldValidators(joiStringWithMainType, field) {
        let joiStringWithAllValidators = joiStringWithMainType;
        const { isRequired, isNullable } = field;
        if (isRequired) {
            joiStringWithAllValidators += '.required()';
        }
        if (isNullable) {
            joiStringWithAllValidators += '.allow(null)';
        }
        return joiStringWithAllValidators;
    }
    wrapWithObject({ joiStringFields, isArray = true, forData = false, }) {
        let wrapped = '{';
        wrapped += '\n';
        wrapped += isArray
            ? '  ' + joiStringFields.join(',\r\n')
            : '  ' + joiStringFields;
        wrapped += '\n';
        wrapped += forData ? '  ' + '}' : '}';
        return wrapped;
    }
    getImportJoi() {
        let joiImportStatement = "import Joi from 'joi';";
        joiImportStatement += '\n';
        return joiImportStatement;
    }
    getImportsForSchemaObjects() {
        let imports = this.getImportJoi();
        imports += this.getAllSchemaImports();
        imports += '\n\n';
        return imports;
    }
    getImportsForSchemas(additionalImports) {
        let imports = this.getImportJoi();
        imports += [...additionalImports].join(';\r\n');
        imports += '\n\n';
        return imports;
    }
    addExportSchemaObject(schema) {
        return `export const ${this.name}SchemaObject = ${schema}`;
    }
    addExportSchema(schema, name) {
        return `export const ${name}Schema = ${schema}`;
    }
    getFinalForm(joiStringFields) {
        return `${this.getImportsForSchemaObjects()}${this.addExportSchemaObject(this.wrapWithObject({ joiStringFields }))}`;
    }
    async printSchemaObjects() {
        var _a;
        const joiStringFields = ((_a = this.fields) !== null && _a !== void 0 ? _a : [])
            .map((field) => {
            const value = this.getSchemaObjectLine(field);
            return value;
        })
            .flatMap((item) => item)
            .filter((item) => item && item.length > 0)
            .map((item) => {
            const [joiStringWithMainType, field, skipValidators] = item;
            const value = skipValidators
                ? joiStringWithMainType
                : this.getFieldValidators(joiStringWithMainType, field);
            return value;
        });
        await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/objects/${this.name}.schema.ts`), this.getFinalForm(joiStringFields));
    }
    async printModelSchemas() {
        var _a;
        for (const model of (_a = this.modelOperations) !== null && _a !== void 0 ? _a : []) {
            const { model: modelName, findUnique, findFirst, findMany, create, update, deleteMany, updateMany, upsert, aggregate, groupBy, } = model;
            if (findUnique) {
                const imports = [
                    `import { ${modelName}WhereUniqueInputSchemaObject } from './objects/${modelName}WhereUniqueInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${findUnique}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereUniqueInputSchemaObject) }).required()`, `${modelName}FindUnique`)}`);
            }
            if (findFirst) {
                const imports = [
                    `import { ${modelName}WhereInputSchemaObject } from './objects/${modelName}WhereInput.schema'`,
                    `import { ${modelName}OrderByWithRelationInputSchemaObject } from './objects/${modelName}OrderByWithRelationInput.schema'`,
                    `import { ${modelName}WhereUniqueInputSchemaObject } from './objects/${modelName}WhereUniqueInput.schema'`,
                    `import { ${modelName}ScalarFieldEnumSchema } from './enums/${modelName}ScalarFieldEnum.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${findFirst}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereInputSchemaObject), orderBy: Joi.object().keys(${modelName}OrderByWithRelationInputSchemaObject), cursor: Joi.object().keys(${modelName}WhereUniqueInputSchemaObject), take: Joi.number(), skip: Joi.number(), distinct: Joi.array().items(${modelName}ScalarFieldEnumSchema) }).required()`, `${modelName}FindFirst`)}`);
            }
            if (findMany) {
                const imports = [
                    `import { ${modelName}WhereInputSchemaObject } from './objects/${modelName}WhereInput.schema'`,
                    `import { ${modelName}OrderByWithRelationInputSchemaObject } from './objects/${modelName}OrderByWithRelationInput.schema'`,
                    `import { ${modelName}WhereUniqueInputSchemaObject } from './objects/${modelName}WhereUniqueInput.schema'`,
                    `import { ${modelName}ScalarFieldEnumSchema } from './enums/${modelName}ScalarFieldEnum.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${findMany}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereInputSchemaObject), orderBy: Joi.object().keys(${modelName}OrderByWithRelationInputSchemaObject), cursor: Joi.object().keys(${modelName}WhereUniqueInputSchemaObject), take: Joi.number(), skip: Joi.number(), distinct: Joi.array().items(${modelName}ScalarFieldEnumSchema)  }).required()`, `${modelName}FindMany`)}`);
            }
            if (create) {
                const imports = [
                    `import { ${modelName}CreateInputSchemaObject } from './objects/${modelName}CreateInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${create}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ data: Joi.object().keys(${modelName}CreateInputSchemaObject)  }).required()`, `${modelName}Create`)}`);
            }
            if (model.delete) {
                const imports = [
                    `import { ${modelName}WhereUniqueInputSchemaObject } from './objects/${modelName}WhereUniqueInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${model.delete}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereUniqueInputSchemaObject)  }).required()`, `${modelName}DeleteOne`)}`);
            }
            if (deleteMany) {
                const imports = [
                    `import { ${modelName}WhereInputSchemaObject } from './objects/${modelName}WhereInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${deleteMany}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereInputSchemaObject)  }).required()`, `${modelName}DeleteMany`)}`);
            }
            if (update) {
                const imports = [
                    `import { ${modelName}UpdateInputSchemaObject } from './objects/${modelName}UpdateInput.schema'`,
                    `import { ${modelName}WhereUniqueInputSchemaObject } from './objects/${modelName}WhereUniqueInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${update}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ data: Joi.object().keys(${modelName}UpdateInputSchemaObject), where: Joi.object().keys(${modelName}WhereUniqueInputSchemaObject)  }).required()`, `${modelName}UpdateOne`)}`);
            }
            if (updateMany) {
                const imports = [
                    `import { ${modelName}UpdateManyMutationInputSchemaObject } from './objects/${modelName}UpdateManyMutationInput.schema'`,
                    `import { ${modelName}WhereInputSchemaObject } from './objects/${modelName}WhereInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${updateMany}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ data: Joi.object().keys(${modelName}UpdateManyMutationInputSchemaObject), where: Joi.object().keys(${modelName}WhereInputSchemaObject)  }).required()`, `${modelName}UpdateMany`)}`);
            }
            if (upsert) {
                const imports = [
                    `import { ${modelName}WhereUniqueInputSchemaObject } from './objects/${modelName}WhereUniqueInput.schema'`,
                    `import { ${modelName}CreateInputSchemaObject } from './objects/${modelName}CreateInput.schema'`,
                    `import { ${modelName}UpdateInputSchemaObject } from './objects/${modelName}UpdateInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${upsert}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereUniqueInputSchemaObject), data: Joi.object().keys(${modelName}CreateInputSchemaObject), update: Joi.object().keys(${modelName}UpdateInputSchemaObject)  }).required()`, `${modelName}Upsert`)}`);
            }
            if (aggregate) {
                const imports = [
                    `import { ${modelName}WhereInputSchemaObject } from './objects/${modelName}WhereInput.schema'`,
                    `import { ${modelName}OrderByWithRelationInputSchemaObject } from './objects/${modelName}OrderByWithRelationInput.schema'`,
                    `import { ${modelName}WhereUniqueInputSchemaObject } from './objects/${modelName}WhereUniqueInput.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${aggregate}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereInputSchemaObject), orderBy: Joi.object().keys(${modelName}OrderByWithRelationInputSchemaObject), cursor: Joi.object().keys(${modelName}WhereUniqueInputSchemaObject), take: Joi.number(), skip: Joi.number()  }).required()`, `${modelName}Aggregate`)}`);
            }
            if (groupBy) {
                const imports = [
                    `import { ${modelName}WhereInputSchemaObject } from './objects/${modelName}WhereInput.schema'`,
                    `import { ${modelName}OrderByWithAggregationInputSchemaObject } from './objects/${modelName}OrderByWithAggregationInput.schema'`,
                    `import { ${modelName}ScalarWhereWithAggregatesInputSchemaObject } from './objects/${modelName}ScalarWhereWithAggregatesInput.schema'`,
                    `import { ${modelName}ScalarFieldEnumSchema } from './enums/${modelName}ScalarFieldEnum.schema'`,
                ];
                await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/${groupBy}.schema.ts`), `${this.getImportsForSchemas(imports)}${this.addExportSchema(`Joi.object().keys({ where: Joi.object().keys(${modelName}WhereInputSchemaObject), orderBy: Joi.object().keys(${modelName}OrderByWithAggregationInputSchemaObject), having: Joi.object().keys(${modelName}ScalarWhereWithAggregatesInputSchemaObject), take: Joi.number(), skip: Joi.number(), by: Joi.array().items(${modelName}ScalarFieldEnumSchema).required()  }).required()`, `${modelName}GroupBy`)}`);
            }
        }
    }
    async printEnumSchemas() {
        var _a;
        for (const enumType of (_a = this.enumTypes) !== null && _a !== void 0 ? _a : []) {
            const { name, values } = enumType;
            await (0, writeFileSafely_1.writeFileSafely)(path_1.default.join(Transformer.outputPath, `schemas/enums/${name}.schema.ts`), `${this.getImportJoi()}\n${this.addExportSchema(`Joi.string().valid(...${JSON.stringify(values)})`, `${name}`)}`);
        }
    }
}
exports.default = Transformer;
//# sourceMappingURL=transformer.js.map